package com.simuwang.daq.components.report.parser.pdf;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.Constants;
import com.simuwang.base.common.exception.APIException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.ReportBaseInfoDTO;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.simuwang.base.pojo.dto.report.ReportFundInfoDTO;
import com.simuwang.base.pojo.dto.report.ReportParserParams;
import com.simuwang.daq.components.CustomPDFTextStripper;
import com.simuwang.daq.components.report.parser.AbstractReportParser;
import org.apache.pdfbox.Loader;
import org.apache.pdfbox.io.RandomAccessReadBufferedFile;
import org.apache.pdfbox.pdmodel.PDDocument;
import technology.tabula.*;
import technology.tabula.extractors.SpreadsheetExtractionAlgorithm;

import java.io.IOException;
import java.util.Calendar;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author wangzaijun
 * @date 2024/9/29 16:45
 * @description pdf格式的报告解析抽象类
 */
public abstract class AbstractPDReportParser<T extends ReportData> extends AbstractReportParser<T> {
    public AbstractPDReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws IOException {
        this.init();
        // 解析报告名称和表格
        String reportName = null;
        try (PDDocument document = Loader.loadPDF(new RandomAccessReadBufferedFile(params.getFilepath()))) {
            CustomPDFTextStripper stripper = new CustomPDFTextStripper(document, 0);
            stripper.setSortByPosition(true);
            String text = stripper.getText(document).replace(Constants.WATERMARK_REPLACE, Constants.EMPTY);
            List<String> textList = StrUtil.split(text, System.lineSeparator());
            textList.removeIf(StrUtil::isBlank);
            if (CollUtil.isNotEmpty(textList)) {
                reportName = this.matchReportName(textList.get(0));
                if (StrUtil.isBlank(reportName)) {
                    throw new APIException("未匹配到报告名称");
                }
            }
            // 解析所有表格
            SpreadsheetExtractionAlgorithm extractionAlgorithm = new SpreadsheetExtractionAlgorithm();
            PageIterator pageIterator = new CustomObjectExtractor(document).extract();
            while (pageIterator.hasNext()) {
                Page page = pageIterator.next();
                List<Table> tables = extractionAlgorithm.extract(page);
                this.initTableInfo(tables);
            }
        }
        // 解析报告中主体基金的基本信息
        ReportFundInfoDTO reportFundInfo = this.parseFundInfo(params);
        // 解析其他表格信息并且设置结果字段
        return this.parseExtInfoAndSetData(reportName, reportFundInfo, params);
    }

    protected abstract void initTableInfo(List<Table> tables);

    protected abstract ReportFundInfoDTO parseFundInfo(ReportParserParams params);

    protected abstract T parseExtInfoAndSetData(String reportName, ReportFundInfoDTO fundInfo, ReportParserParams params);

    /**
     * 构建报告基本信息
     *
     * @param fileId     文件id
     * @param reportName 报告名称
     * @return /
     */
    protected ReportBaseInfoDTO buildReportInfo(Integer fileId, String reportName) {
        ReportBaseInfoDTO reportInfo = new ReportBaseInfoDTO();
        reportInfo.setFileId(fileId);
        reportInfo.setReportName(reportName);
        reportInfo.setReportType(this.matchReportType(reportName));
        reportInfo.setReportDate(this.matchReportDate(reportName));
        return reportInfo;
    }

    /**
     * 匹配报告名称
     *
     * @param text 文本内容
     * @return /
     */
    private String matchReportName(String text) {
        if (StrUtil.isBlank(text)) {
            return null;
        }
        // 编译正则表达式模式
        Pattern pat1 = Pattern.compile(".+?报([告表])?\\d{4}(\\.?\\d{1,2}(\\.?\\d{2})?)?");
        Pattern pat2 = Pattern.compile("私募.*披露年度报[告表](（\\d{4}-\\d{2}-\\d{2}至\\d{4}-\\d{2}-\\d{2}）)?");
        Pattern pat3 = Pattern.compile(".+?报([告表])?\\d{4}-\\d{2}-\\d{2}至\\d{4}-\\d{2}-\\d{2}?");

        // 创建Matcher对象
        Matcher matcher1 = pat1.matcher(text);
        Matcher matcher2 = pat2.matcher(text);
        Matcher matcher3 = pat3.matcher(text);

        // 尝试匹配
        String reportName;
        if (matcher1.find()) {
            reportName = matcher1.group();
        } else if (matcher2.find()) {
            reportName = matcher2.group();
        } else if (matcher3.find()) {
            reportName = matcher3.group();
        } else {
            reportName = text;
        }
        return reportName.replace("（", "(").replace("）", ")").trim();
    }

    /**
     * 匹配报告日期
     *
     * @param string 文本内容
     * @return 报告日期
     */
    private String matchReportDate(String string) {
        if (string == null) {
            return null;
        }

        // 编译正则表达式模式
        Pattern pat1 = Pattern.compile("(2\\d{3}).*([一二三四1234])季度");  // 2023年XXX3季度
        Pattern pat2 = Pattern.compile("\\d{4}-\\d{2}-\\d{2}");  // 2023-12-31
        Pattern pat3 = Pattern.compile("(2\\d{3})年年度");  // 2023年年度
        Pattern pat4 = Pattern.compile("(\\d{4})年(\\d{1,2})月");  // 2023年12月

        // 创建Matcher对象
        Matcher matcher1 = pat1.matcher(string);
        Matcher matcher2 = pat2.matcher(string);
        Matcher matcher3 = pat3.matcher(string);
        Matcher matcher4 = pat4.matcher(string);

        // 尝试匹配
        if (matcher1.find()) {
            String year = matcher1.group(1);
            String quarter = matcher1.group(2);
            return switch (quarter) {
                case "一", "1" -> year + "-03-31";
                case "二", "2" -> year + "-06-30";
                case "三", "3" -> year + "-09-30";
                case "四", "4" -> year + "-12-31";
                default -> null;
            };
        } else if (matcher2.find()) {
            return matcher2.group();
        } else if (matcher3.find()) {
            return matcher3.group(1) + "-12-31";
        } else if (matcher4.find()) {
            String year = matcher4.group(1);
            String month = matcher4.group(2);
            int lastDayOfMonth = getLastDayOfMonth(Integer.parseInt(year), Integer.parseInt(month));
            return year + "-" + padZero(month) + "-" + padZero(lastDayOfMonth + "");
        } else {
            return null;
        }
    }

    /**
     * 匹配报告类型，如“季度”、“年度”
     *
     * @param string 输入字符串
     * @return 匹配到的报告类型子字符串，如果没有匹配到则返回null
     */
    private String matchReportType(String string) {
        if (string == null) {
            return null;
        }

        // 编译正则表达式模式
        Pattern pattern = Pattern.compile("月|季度|年度");

        // 创建Matcher对象
        Matcher matcher = pattern.matcher(string);

        // 尝试匹配
        if (matcher.find()) {
            return matcher.group();
        } else {
            return null;
        }
    }

    private int getLastDayOfMonth(int year, int month) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month - 1); // Calendar.MONTH 是从0开始的
        return calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
    }

    private String padZero(String number) {
        return String.format("%02d", Integer.parseInt(number));
    }
}
